﻿Type=Class
Version=2.00
@EndOfDesignText@
' Class module
' CheckList v1.6
Sub Class_Globals
	Public BackgroundColor As Int
	Public CheckedColor As Int
	Public DividerColor As Int
	Public ExtensionColor As Int
	Private sv As ScrollView
	Private svHeight As Int
	Private ExtensionIndex As Int
	Private ExtensionView As View
	Private CallbackMod As Object
	Private sub_Check, sub_Click, sub_LongClick As String
	Private ModeCheck As Boolean
	Private BlockCheckEvent As Boolean
	Private PressedPanel As Panel
	Private PressedDrawable As Object
	Private OldBackground As Object
	Private DividerSize As Int
	' Constants
	Private ChkBoxSize  As Int: ChkBoxSize  = 40dip
	Private BitmapSize As Int: BitmapSize = 50dip
	Private LabelHeight  As Int: LabelHeight  = 25dip
	Private PanelHeight As Int: PanelHeight = 2 * LabelHeight
End Sub

' Initializes the ScrollView.
' Module: Callback module (Me)
' svList: ScrollView used to display the list
' subCheck, subClick, subLongClick: Name of subs handling events (if subClick="", clicks on items check/uncheck the checkbox)
' DividerHeight: Height of the line separating items (0 = no line)
Public Sub Initialize(Module As Object, svList As ScrollView, subCheck As String, subClick As String, subLongClick As String, DividerHeight As Int)
	sv = svList
	For i = sv.Panel.NumberOfViews-1 To 0 Step -1
		sv.Panel.RemoveViewAt(i)
	Next
	sv.Panel.Height = 0
	svHeight = 0
	CallbackMod = Module
	sub_Check = subCheck
	sub_Click = subClick
	sub_LongClick = subLongClick
	ModeCheck = (sub_Click = "")
	BlockCheckEvent = False
	DividerSize = DividerHeight
	DividerColor = Colors.ARGB(128, 200, 200, 200)
	BackgroundColor = Colors.Transparent
	CheckedColor = Colors.Transparent
	ExtensionColor = Colors.Transparent
	ExtensionIndex = -1
	ExtensionView = Null
	Dim r As Reflector
	r.Target = r.GetContext
	r.Target = r.RunMethod("getResources")
	r.Target = r.RunMethod("getSystem")
	Dim ID_PressedDrawable As Int
	ID_PressedDrawable = r.RunMethod4("getIdentifier", Array As Object("list_selector_background_pressed", "drawable", "android"), _
																		Array As String("java.lang.String", "java.lang.String", "java.lang.String"))
	r.Target = r.GetContext
	r.Target = r.RunMethod("getResources")
	PressedDrawable = r.RunMethod2("getDrawable", ID_PressedDrawable, "java.lang.int")
End Sub

Private Sub CreateScaledBitmap(Original As Bitmap, Width As Int, Height As Int) As Bitmap
	' Resize the picture
	Dim r As Reflector
	Dim b As Bitmap
	b = r.RunStaticMethod("android.graphics.Bitmap", "createScaledBitmap", _
			Array As Object(Original, Width, Height, True), _
			Array As String("android.graphics.Bitmap", "java.lang.int", "java.lang.int", "java.lang.boolean"))
	Return b
End Sub

' Resizes the bitmap to fit the given dimensions
Public Sub AdjustBitmap(bmp As Bitmap, Width, Height As Int) As Bitmap
	Dim RatioBmp, RatioIV As Float
	RatioBmp = bmp.Width / bmp.Height
	RatioIV = Width / Height
	Dim Diviseur As Float
	If RatioIV > RatioBmp Then
		Diviseur = bmp.Height / Height
		bmp = CreateScaledBitmap(bmp, Round(bmp.Width / Diviseur / Density), Round(Height / Density))
	Else
		Diviseur = bmp.Width / Width
		bmp = CreateScaledBitmap(bmp, Round(Width / Density), Round(bmp.Height / Diviseur / Density))
	End If
	Return bmp
End Sub

' Adds a item (the created panel contains a checkbox, an optional bitmap and one or two labels)
' Set Text2 to "" if you don't want a second label and Image to Null if there's no bitmap
Public Sub AddItem(ID As Int, Checked As Boolean, Text1 As String, Text2 As String, Image As Bitmap)
	Dim pnl As Panel: pnl.Initialize("")
	Dim chk As CheckBox:	chk.Initialize("chbSV")
	pnl.Tag = ID
	pnl.AddView(chk, 0, 0, ChkBoxSize, PanelHeight)
	BlockCheckEvent = True
	chk.Checked = Checked
	BlockCheckEvent = False
	Dim LargeurLabel As Int, PosX As Int
	LargeurLabel = 150%x
	If Not(Image.IsInitialized) Then
		PosX = ChkBoxSize
	Else
		Dim iv As ImageView: iv.Initialize("")
		Dim bmp As Bitmap
		bmp = Image
		If NumberFormat(bmp.Width / bmp.Height, 1, 2) = NumberFormat(1, 1, 2) Then
			' Aspect ratio nearly equal -> filling
			iv.Gravity = Gravity.FILL
		Else
			' Different aspect ratio -> resizing
			bmp = AdjustBitmap(bmp, BitmapSize, BitmapSize)
			iv.Gravity = Gravity.NO_GRAVITY
		End If
		iv.Bitmap = bmp
		pnl.AddView(iv, ChkBoxSize, 0, BitmapSize, BitmapSize)
		PosX = ChkBoxSize + BitmapSize + 5dip
	End If
	Dim lbl1 As Label: lbl1.Initialize("")
	lbl1.Gravity = Gravity.CENTER_VERTICAL
	lbl1.Text = Text1
	lbl1.TextColor = Colors.White
	lbl1.TextSize = 18
	lbl1.Typeface = Typeface.DEFAULT_BOLD
	If Text2 = "" Then
		pnl.AddView(lbl1, PosX, 0, LargeurLabel, PanelHeight)
	Else
		pnl.AddView(lbl1, PosX, 0, LargeurLabel, LabelHeight)
		Dim lbl2 As Label: lbl2.Initialize("")
		lbl2.Gravity = Gravity.TOP
		lbl2.Text = Text2
		lbl2.TextColor = Colors.LightGray
		lbl2.TextSize = 16
		lbl2.Typeface = Typeface.DEFAULT
		pnl.AddView(lbl2, PosX, LabelHeight, LargeurLabel, LabelHeight)
	End If
	AddToSV(pnl, PanelHeight, True)
End Sub

' Adds a item without checkbox (the created panel contains an optional bitmap and one or two labels)
' Set Text2 to "" if you don't want a second label and Image to Null if there's no bitmap
Public Sub AddItemNoChkbx(ID As Int, Text1 As String, Text2 As String, Image As Bitmap)
	Dim pnl As Panel: pnl.Initialize("")
	pnl.Tag = ID
	Dim LargeurLabel As Int, PosX As Int
	LargeurLabel = 150%x
	If Image.IsInitialized Then
		Dim iv As ImageView: iv.Initialize("")
		Dim bmp As Bitmap
		bmp = Image
		If NumberFormat(bmp.Width / bmp.Height, 1, 2) = NumberFormat(1, 1, 2) Then
			' Aspect ratio nearly equal -> filling
			iv.Gravity = Gravity.FILL
		Else
			' Different aspect ratio -> resizing
			bmp = AdjustBitmap(bmp, BitmapSize, BitmapSize)
			iv.Gravity = Gravity.NO_GRAVITY
		End If
		iv.Bitmap = bmp
		pnl.AddView(iv, 0, 0, BitmapSize, BitmapSize)
		PosX = BitmapSize + 5dip
	End If
	Dim lbl1 As Label: lbl1.Initialize("")
	lbl1.Gravity = Gravity.CENTER_VERTICAL
	lbl1.Text = Text1
	lbl1.TextColor = Colors.White
	lbl1.TextSize = 18
	lbl1.Typeface = Typeface.DEFAULT_BOLD
	If Text2 = "" Then
		pnl.AddView(lbl1, PosX, 0, LargeurLabel, PanelHeight)
	Else
		pnl.AddView(lbl1, PosX, 0, LargeurLabel, LabelHeight)
		Dim lbl2 As Label: lbl2.Initialize("")
		lbl2.Gravity = Gravity.TOP
		lbl2.Text = Text2
		lbl2.TextColor = Colors.LightGray
		lbl2.TextSize = 16
		lbl2.Typeface = Typeface.DEFAULT
		pnl.AddView(lbl2, PosX, LabelHeight, LargeurLabel, LabelHeight)
	End If
	AddToSV(pnl, PanelHeight, True)
End Sub

' Adds a custom item (a panel holding your custom layout)
Public Sub AddCustomItem(ID As Int, pnl As Panel, pnlHeight As Int)
	pnl.Tag = ID
	AddToSV(pnl, pnlHeight, True)
End Sub

Private Sub AddToSV(pnl As Panel, pnlHeight As Int, WithEvents As Boolean)
	' Ajout du séparateur
	If DividerSize > 0 AND svHeight > 0 Then
		For i = 0 To pnl.NumberOfViews - 1
			pnl.GetView(i).Top = pnl.GetView(i).Top + DividerSize
		Next
		Dim pnlDiv As Panel
		pnlDiv.Initialize("")
		pnlDiv.Color = DividerColor
		pnl.AddView(pnlDiv, 0, 0, getWidth, DividerSize)
		pnlHeight = pnlHeight + DividerSize
	End If

	' Ajout du panel et agrandissement de la hauteur du ScrollView
	sv.Panel.AddView(pnl, 0, svHeight, getWidth, pnlHeight)
	svHeight = svHeight + pnl.Height

	' Affectation des événements Touch, Click et LongClick
	If Not(WithEvents) Then Return
	Dim r As Reflector
	r.Target = pnl
	r.SetOnTouchListener("pnlSVTouch")
	If sub_Click <> "" Then r.SetOnClickListener("pnlSVClick")
	If sub_LongClick <> "" Then r.SetOnLongClickListener("pnlSVLongClick")
End Sub

' Gets the ScrollView width
' In some containers like TabHosts, the width property returns -1, so this function uses a different method to get it
Public Sub getWidth
	If sv.Width < 0 Then
		Dim r As Reflector, Largeur As Int
		r.Target = sv
		Largeur = r.RunMethod("getWidth")
		If Largeur = 0 Then
			DoEvents
			Largeur = r.RunMethod("getWidth")
		End If
		Return Largeur		
	Else
		Return sv.Width
	End If
End Sub

' Gets the ScrollView height
Public Sub getHeight
	If sv.Height < 0 Then
		Dim r As Reflector, Hauteur As Int
		r.Target = sv
		Hauteur = r.RunMethod("getHeight")
		If Hauteur = 0 Then
			DoEvents
			Hauteur = r.RunMethod("getHeight")
		End If
		Return Hauteur		
	Else
		Return sv.Height
	End If
End Sub

' Returns the number of items
Public Sub NumberOfItems
	If sv.IsInitialized Then
		Return sv.Panel.NumberOfViews
	Else
		Return 0
	End If
End Sub

' Adds a header
Public Sub AddHeader(Text As String)
	Dim pnlH As Panel: pnlH.Initialize("")
	Dim lblB As Label: lblB.Initialize("")
	Dim cdH As ColorDrawable
	cdH.Initialize(Colors.ARGB(100, 30, 30, 30), 10)
	lblB.Background = cdH
	lblB.Text = "   " & Text
	lblB.TextColor = Colors.Yellow
	lblB.TextSize = 18
	lblB.Typeface = Typeface.DEFAULT_BOLD
	lblB.Gravity = Gravity.CENTER_VERTICAL
	pnlH.AddView(lblB, 0, 0, getWidth, PanelHeight)
	AddToSV(pnlH, PanelHeight, False)
End Sub

' Resizes the ScrollView panel height
' (must be called when all items are added)
Public Sub ResizePanel()
	CollapseItem
	sv.Panel.Height = svHeight
End Sub

Private Sub CommonJump(NewPosition As Int)
	' We remove a few dips for the fading edge
	NewPosition = Max(0, NewPosition - 5dip)

	' We wait until the scrolling animation ends
	Dim PreviousPosition As Int: PreviousPosition = -1
	Do Until PreviousPosition = sv.ScrollPosition
		PreviousPosition = sv.ScrollPosition
		sv.ScrollPosition = NewPosition
		DoEvents
	Loop
End Sub

' Scrolls the list to the given position
' Returns the panel of the item found
Public Sub JumpToPosition(Position As Int) As Panel
	If sv.IsInitialized Then
		Position = Max(Position, 0)
		Position = Min(Position, NumberOfItems - 1)
		Dim TotalHeight As Int
		For i = 0 To NumberOfItems - 1
			Dim pnl As Panel
			pnl = sv.Panel.GetView(i)
			If Position = i Then
				CommonJump(TotalHeight)
				Return pnl
			End If
			TotalHeight = TotalHeight + pnl.Height
		Next
	End If
	Return Null
End Sub

' Scrolls the list to the item with the given ID
' Returns the panel of the item found
Public Sub JumpToItem(ID As Int) As Panel
	If sv.IsInitialized Then
		Dim TotalHeight As Int
		For i = 0 To NumberOfItems - 1
			Dim pnl As Panel
			pnl = sv.Panel.GetView(i)
			If pnl.Tag <> Null Then
				If pnl.Tag = ID Then
					CommonJump(TotalHeight)
					Return pnl
				End If
			End If
			TotalHeight = TotalHeight + pnl.Height
		Next
	End If
	Return Null
End Sub

' Scrolls the list to the given view
' It's faster than JumpToPosition and JumpToItem
Public Sub JumpToView(MyView As View) As Panel
	If sv.IsInitialized Then
		Dim pnl As Panel
		pnl = GetParentPanel(MyView)
		CommonJump(pnl.Top)
		Return pnl
	End If
End Sub

Private Sub GetParentPanel(MyView As View) As View
	Dim r As Reflector
	Dim v, Parent As View
	r.Target = MyView
	v = r.Target
	Parent = r.RunMethod("getParent")
	Do While Parent <> sv.Panel
		r.Target = Parent
		v = r.Target
		Parent = r.RunMethod("getParent")
	Loop
	Return v
End Sub

' Returns the panel containing the given view.
Public Sub FindPanelContaining(MyView As View) As Panel
	If sv.IsInitialized Then
		Return GetParentPanel(MyView)
	End If
End Sub

Private Sub IsCheckBox(MyView As View) As Boolean
	Return (GetType(MyView) = "android.widget.CheckBox")
End Sub

Private Sub IsLabel(MyView As View) As Boolean
	Return (GetType(MyView) = "android.widget.TextView")
End Sub

' Creates a list with ID of checked items
Public Sub CheckedList As List
	Dim ListeCoches As List
	ListeCoches.Initialize 
	If sv.IsInitialized Then
		For i = 0 To sv.Panel.NumberOfViews-1
			Dim pnl As Panel
			pnl = sv.Panel.GetView(i)
			For p = 0 To pnl.NumberOfViews - 1
				If IsCheckBox(pnl.GetView(p)) Then
					Dim chk As CheckBox
					chk = pnl.GetView(p)
					If chk.Checked Then ListeCoches.Add(pnl.Tag)
					Exit
				End If
			Next
		Next
	End If
	Return ListeCoches
End Sub

' Checks/unchecks all checkboxes
Public Sub CheckAllBoxes(Checked As Boolean)
	BlockCheckEvent = True 'We don't want to trigger a CheckedChange event for each checkbox
	Dim pnl As Panel, chb As CheckBox
	For i = 0 To sv.Panel.NumberOfViews - 1
		pnl = sv.Panel.GetView(i)
		For p = 0 To pnl.NumberOfViews - 1
			If IsCheckBox(pnl.GetView(p)) Then
				chb = pnl.GetView(p)
				chb.Checked = Checked
				Exit
			End If
		Next
	Next
	BlockCheckEvent = False
End Sub

Private Sub GetColor(pnl As Panel, Extended As Boolean)
	For p = 0 To pnl.NumberOfViews - 1
		If IsCheckBox(pnl.GetView(p)) Then
			Dim chk As CheckBox
			chk = pnl.GetView(p)
			If chk.Checked Then Return CheckedColor
			Exit
		End If
	Next
	If Extended Then
		Return ExtensionColor
	Else
		Return BackgroundColor
	End If
End Sub

' Adds an extra content (panel, label, edittext, imageview...) under an item
Public Sub ExtendItem(ItemID As Int, AddedContent As View, ContentHeight As Int)
	If AddedContent.IsInitialized Then
		If HasExtraContent Then CollapseItem
		Dim MoveFollowingPanels As Boolean
		MoveFollowingPanels = False
		Dim pnl As Panel
		For i = 0 To sv.Panel.NumberOfViews - 1
			pnl = sv.Panel.GetView(i)
			If MoveFollowingPanels Then
				pnl.Top = pnl.Top + ContentHeight
			Else If pnl.Tag = Null Then
				Continue ' It's a header
			Else If pnl.Tag = ItemID Then
				ExtensionIndex = i
				ExtensionView = AddedContent
				pnl.Color = GetColor(pnl, True)
				If IsCheckBox(pnl.GetView(0)) Then
					' If the first view is a checkbox, the extension panel is shifted to the right of this checkbox
					pnl.AddView(AddedContent, ChkBoxSize, pnl.Height, getWidth - ChkBoxSize, ContentHeight)
				Else
					pnl.AddView(AddedContent, 0, pnl.Height, getWidth, ContentHeight)
				End If
				pnl.Height = pnl.Height + ContentHeight
				svHeight = svHeight + ContentHeight
				sv.Panel.Height = svHeight

				Dim ContentBottom As Int
				ContentBottom = pnl.Top + pnl.Height
				If ContentBottom > sv.ScrollPosition + getHeight Then
					' If needed, we scroll to display the added content
					Dim NewPosition As Int
					DoEvents
					NewPosition = sv.ScrollPosition + Min(ContentBottom - sv.ScrollPosition - getHeight, getHeight) + 15dip
					CommonJump(NewPosition)
				End If
				
				AddedContent.RequestFocus
				MoveFollowingPanels = True
			End If
		Next
	End If
End Sub

' Removes the extra content
Public Sub CollapseItem
	If ExtensionIndex <> -1 Then 
		Dim ExtensionHeight As Int
		ExtensionHeight = ExtensionView.Height
		ExtensionView.RemoveView
		Dim pnl As Panel
		pnl = sv.Panel.GetView(ExtensionIndex)
		pnl.Color = GetColor(pnl, False)
		pnl.Height = pnl.Height - ExtensionHeight
		svHeight = svHeight - ExtensionHeight
		sv.Panel.Height = svHeight
		For i = ExtensionIndex + 1 To sv.Panel.NumberOfViews - 1
			pnl = sv.Panel.GetView(i)
			pnl.Top = pnl.Top - ExtensionHeight
		Next
		ExtensionIndex = -1
	End If
	ExtensionView = Null
End Sub

' Checks if there is an item with extra content
Public Sub HasExtraContent
	Return (ExtensionIndex <> -1)
End Sub

' Returns the ID of the item with extra content
' Returns -1 if there's no such item
' -1 could be an ID in your app, so check HasExtraContent before calling this function
Public Sub ExtendedItemID As Int
	If ExtensionView.IsInitialized Then
		Dim pnl As Panel
		pnl = FindPanelContaining(ExtensionView)
		Return pnl.Tag
	Else
		Return -1
	End If
End Sub

' Returns the first label text found for the given ID
' Returns an empty string if not found
Public Sub GetFirstText(ID As Int) As String
	Dim pnl As Panel
	For i = 0 To sv.Panel.NumberOfViews - 1
		pnl = sv.Panel.GetView(i)
		If pnl.Tag = Null Then Continue
		If pnl.Tag = ID Then
			For p = 0 To pnl.NumberOfViews - 1
				If IsLabel(pnl.GetView(p)) Then
					Dim lbl As Label
					lbl = pnl.GetView(p)
					Return lbl.Text
				End If
			Next
			Exit
		End If
	Next
	Return ""
End Sub

Private Sub pnlSVTouch(ViewTag As Object, Action As Int, X As Float, Y As Float, MotionEvent As Object) As Boolean
	Dim pnl As Panel
	pnl = Sender
	If Action <> 2 Then		' <> ACTION_MOVE
		If PressedPanel.IsInitialized Then
			PressedPanel.Background = OldBackground
			PressedPanel = Null
		End If
	End If
	If Action = 0 Then		' = ACTION_DOWN
		OldBackground = pnl.Background
		PressedPanel = pnl
		PressedPanel.Background = PressedDrawable
	Else If Action = 1 Then	' = ACTION_UP
		If ModeCheck Then
			' Check the box
			For p = 0 To pnl.NumberOfViews - 1
				If IsCheckBox(pnl.GetView(p)) Then
					Dim chb As CheckBox
					chb = pnl.GetView(p)
					chb.Checked = Not(chb.Checked)
					Exit
				End If
			Next
		End If
	End If
	Return (sub_Click = "" AND sub_LongClick = "")
End Sub

Private Sub pnlSVClick(ViewTag As Object) As Boolean
	Dim pnl As Panel
	pnl = Sender
	If SubExists(CallbackMod, sub_Click) Then CallSub3(CallbackMod, sub_Click, pnl, pnl.Tag)
	Return True
End Sub

Private Sub pnlSVLongClick(ViewTag As Object) As Boolean
	Dim pnl As Panel
	pnl = Sender
	If SubExists(CallbackMod, sub_LongClick) Then CallSub3(CallbackMod, sub_LongClick, pnl, pnl.Tag)
	Return True
End Sub

Private Sub chbSV_CheckedChange(Checked As Boolean)
	Dim r As Reflector
	Dim chb As CheckBox, pnl As Panel
	chb = Sender
	r.Target = chb
	pnl = r.RunMethod("getParent")
	If Checked Then
		pnl.Color = CheckedColor
	Else
		If HasExtraContent AND ExtendedItemID = pnl.Tag Then
			pnl.Color = ExtensionColor
		Else
			pnl.Color = BackgroundColor
		End If
	End If
	If BlockCheckEvent Then Return
	If sub_Check <> "" Then
		If SubExists(CallbackMod, sub_Check) Then CallSub3(CallbackMod, sub_Check, chb, pnl.Tag)
	End If
End Sub
